"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
var BatchUpdateHost_1 = require("./BatchUpdateHost");
var RecycleStrategy_1 = require("./RecycleStrategy");
var searchCycleCollection_1 = require("./searchCycleCollection");
var event_kit_1 = require("event-kit");
var IS_APP_STORE = "@@__APP_STORE__@@";
var AppStore = /** @class */ (function () {
    function AppStore(storeDeclarers, initStates) {
        this._isInit = false;
        this.prevState = {};
        this.state = {};
        this.stores = {};
        this._emitter = new event_kit_1.Emitter();
        this._storeRegisterMap = {};
        this._recycleStrategy = RecycleStrategy_1.default.Never;
        this._depStoreList = {};
        this.batchUpdater = new BatchUpdateHost_1.default(this);
        if (Array.isArray(storeDeclarers)) {
            this.registerStore.apply(this, __spread(storeDeclarers));
        }
        else {
            initStates = storeDeclarers;
        }
        if (initStates) {
            this.__initStates__ = initStates;
            this.state = initStates;
        }
    }
    // buildStore(storeClass, args, options) {
    //   return buildStore(this, storeClass, args, options);
    // }
    AppStore.prototype.setState = function (state) {
        if (!this._isInit) {
            // The AppStore has not initialilized
            Object.assign(this.state, state);
        }
        else {
            this.state = Object.assign({}, this.state, state);
            this.batchUpdater.requestUpdate();
        }
    };
    AppStore.prototype._sendUpdate = function () {
        this.handleWillChange && this.handleWillChange(this.prevState, this.state);
        this._emitter.emit("did-change", this.state);
        this.prevState = this.state;
    };
    AppStore.prototype.forceUpdate = function () {
        this._sendUpdate();
    };
    AppStore.prototype.handleWillChange = function (prevState, state) { };
    AppStore.prototype.onDidChange = function (callback) {
        return this._emitter.on("did-change", callback);
    };
    AppStore.prototype.preloadStores = function (stores, lifetime) {
        var e_1, _a;
        if (lifetime === void 0) { lifetime = "dynamic"; }
        var allDepList = [];
        try {
            for (var stores_1 = __values(stores), stores_1_1 = stores_1.next(); !stores_1_1.done; stores_1_1 = stores_1.next()) {
                var storeKey = stores_1_1.value;
                var store = this.stores[this._getStoreKey(storeKey)];
                if (!store) {
                    var depList = this._createStoreAndInject(storeKey);
                    allDepList = allDepList.concat(depList);
                    // For preload stores, it need not add reference count.
                    if (lifetime === "dynamic") {
                        this.stores[this._getStoreKey(storeKey)]._decreaseRef();
                    }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (stores_1_1 && !stores_1_1.done && (_a = stores_1.return)) _a.call(stores_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        // Wait for all the stores and the dependencies init.
        var initPromises = [];
        for (var i = allDepList.length - 1; i >= 0; i -= 1) {
            var storeKey = allDepList[i];
            initPromises.push(this.stores[this._getStoreKey(storeKey)]._init());
        }
        return Promise.all(initPromises);
    };
    AppStore.prototype.preloadStaticStores = function () {
        var registerMap = this._storeRegisterMap;
        var preloads = [];
        for (var storeKey in registerMap) {
            if (registerMap[storeKey].options.lifetime === "static") {
                preloads.push(storeKey);
            }
        }
        return this.preloadStores(preloads, "static");
    };
    AppStore.prototype.init = function () {
        this._isInit = true;
        this.prevState = this.state;
        this._cycleCollections = searchCycleCollection_1.default(this._storeRegisterMap);
        if (typeof window === "object") {
            this.preloadStaticStores();
        }
        return this;
    };
    AppStore.prototype.dispose = function () {
        this._emitter.dispose();
        this.prevState = this.state = null;
        for (var key in this.stores) {
            var store = this.stores[key];
            store.dispose();
        }
        this.stores = {};
    };
    AppStore.prototype.onDidChangeRS = function (callback) {
        return this._emitter.on("did-change-rs", callback);
    };
    AppStore.prototype.setRecycleStrategy = function (recycleStrategy) {
        if (this._recycleStrategy !== recycleStrategy) {
            this._recycleStrategy = recycleStrategy;
            if (recycleStrategy === RecycleStrategy_1.default.Urgent) {
                for (var finalStoreKey in this.stores) {
                    var store = this.stores[finalStoreKey];
                    if (store && store.getRefCount() === 0) {
                        var _a = __read(this._parseStoreKey(finalStoreKey), 2), storeKey = _a[0], opts = _a[1];
                        this._disposeStoreAndDeps(storeKey, store, opts);
                    }
                    else if (store && store._recycleStrategy == null && store._disposeSubStores) {
                        store._disposeSubStores();
                    }
                }
            }
            this._emitter.emit("did-change-rs", recycleStrategy);
        }
    };
    /**
     * Register the store into the appStore with storeDeclarer
     * @param storeDeclarer: AnyStoreDeclater
     */
    AppStore.prototype.registerStore = function () {
        var e_2, _a;
        var storeDeclarers = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            storeDeclarers[_i] = arguments[_i];
        }
        try {
            for (var storeDeclarers_1 = __values(storeDeclarers), storeDeclarers_1_1 = storeDeclarers_1.next(); !storeDeclarers_1_1.done; storeDeclarers_1_1 = storeDeclarers_1.next()) {
                var storeDeclarer = storeDeclarers_1_1.value;
                var storeKey = storeDeclarer.options.storeKey;
                if (this._storeRegisterMap[storeKey]) {
                    console.error("Many Stores named " + storeKey + " will be registered, That is not allowed!");
                }
                this._storeRegisterMap[storeKey] = storeDeclarer;
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (storeDeclarers_1_1 && !storeDeclarers_1_1.done && (_a = storeDeclarers_1.return)) _a.call(storeDeclarers_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    /**
     * Request the appStore to create store by the storeKey if this store has exists and add reference to 1
     * Request the appStore to add reference count if this store has exists.
     * @param storeKey
     */
    AppStore.prototype.requestStore = function (storeKey, storeOpts) {
        var finalStoreKey = this._getStoreKey(storeKey, storeOpts);
        var store = this.stores[finalStoreKey];
        if (!store) {
            var depList = this._createStoreAndInject(storeKey, storeOpts);
            // Invoke the depList's willInit, the dependency store's willInit will be invoked first
            // for (let i = depList.length - 1; i >= 0; i -= 1) {
            //   let storeKey = depList[i];
            //   this.stores[storeKey].willInit();
            // }
            // Invoke the depList's init, the dependency store's init will be invoked first
            for (var i = depList.length - 1; i >= 0; i -= 1) {
                var storeKey_1 = depList[i];
                this.stores[this._getStoreKey(storeKey_1, storeOpts)]._init();
            }
            store = this.stores[finalStoreKey];
        }
        else {
            store._addRef();
        }
        return store;
    };
    /**
     * Release the store reference count with name of storeKey, if the reference count decrease to 0, then the store destroy.
     * @param storeKey
     */
    AppStore.prototype.releaseStore = function (storeKey, storeOpts) {
        var e_3, _a, e_4, _b;
        var finalStoreKey = this._getStoreKey(storeKey, storeOpts);
        var store = this.stores[finalStoreKey];
        if (!store)
            return;
        // store._decreaseRef();
        // If the store will be disposed, then we check if we can dispose or not
        if (store.getRefCount() === 1) {
            try {
                for (var _c = __values(this._cycleCollections), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var cycleSet = _d.value;
                    if (cycleSet.has(storeKey)) {
                        try {
                            for (var cycleSet_1 = (e_4 = void 0, __values(cycleSet)), cycleSet_1_1 = cycleSet_1.next(); !cycleSet_1_1.done; cycleSet_1_1 = cycleSet_1.next()) {
                                var iKey = cycleSet_1_1.value;
                                iKey = this._getStoreKey(iKey, storeOpts);
                                if (this.stores[iKey].getRefCount() > 1) {
                                    this.stores[iKey]._decreaseRef();
                                    return;
                                }
                            }
                        }
                        catch (e_4_1) { e_4 = { error: e_4_1 }; }
                        finally {
                            try {
                                if (cycleSet_1_1 && !cycleSet_1_1.done && (_b = cycleSet_1.return)) _b.call(cycleSet_1);
                            }
                            finally { if (e_4) throw e_4.error; }
                        }
                    }
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
                }
                finally { if (e_3) throw e_3.error; }
            }
            // console.log("Not found circle dependency can decrease");
            store._decreaseRef();
            // If all of the cycle collections cannot find the ref count > 1, then we can dispose all of the dependency stores
            // breadth-first search all the dependency store and derease the reference count, If this store's count decrease to 0, dispose it.
            if (this._recycleStrategy === RecycleStrategy_1.default.Urgent) {
                this._disposeStoreAndDeps(storeKey, store, storeOpts);
            }
        }
        else {
            store._decreaseRef();
        }
    };
    AppStore.prototype._createStoreAndInject = function (storeKey, storeOpts) {
        var e_5, _a, e_6, _b;
        // Search all of the dependency stores for this `storeKey` with Breadth first search
        var depList = [storeKey];
        for (var i = 0; i < depList.length; i += 1) {
            var curStoreKey = depList[i];
            var storeInfo = this._storeRegisterMap[curStoreKey];
            if (!storeInfo) {
                console.error("The request store " + curStoreKey + " is not registered!");
                continue;
            }
            var depNames = storeInfo.depStoreNames || [];
            var filterDepNames = [];
            try {
                for (var depNames_1 = (e_5 = void 0, __values(depNames)), depNames_1_1 = depNames_1.next(); !depNames_1_1.done; depNames_1_1 = depNames_1.next()) {
                    var depName = depNames_1_1.value;
                    // The depName is not registered
                    if (!this._storeRegisterMap[depName]) {
                        console.error("The dependency store " + depName + " is not registered!");
                        continue;
                    }
                    var depStore = this.stores[this._getStoreKey(depName, storeOpts)];
                    // If the dependency store has exists, then this store don't need to appear in the depList
                    // And if this dependency is the recursive dependency, then we need to remove it from the depList
                    if (depStore) {
                        // This dependency store is exists, then we need add the reference count.
                        depStore._addRef();
                    }
                    else if (depList.indexOf(depName) === -1) {
                        filterDepNames.push(depName);
                    }
                }
            }
            catch (e_5_1) { e_5 = { error: e_5_1 }; }
            finally {
                try {
                    if (depNames_1_1 && !depNames_1_1.done && (_a = depNames_1.return)) _a.call(depNames_1);
                }
                finally { if (e_5) throw e_5.error; }
            }
            depList.splice.apply(depList, __spread([depList.length, 0], filterDepNames));
        }
        // Create all the dependency stores and add the reference count
        for (var i = depList.length - 1; i >= 0; i -= 1) {
            var storeKey_2 = depList[i];
            // let storeInfo = this._storeRegisterMap[storeKey];
            var curStore = this._storeRegisterMap[storeKey_2].create(this);
            curStore._addRef();
            // this.stores[this._getStoreKey(storeKey, storeOpts)] = curStore;
            this._createStore(storeKey_2, curStore, storeOpts);
        }
        // Inject all of the dependency stores for depList
        for (var i = depList.length - 1; i >= 0; i -= 1) {
            var storeKey_3 = depList[i];
            var storeInfo = this._storeRegisterMap[storeKey_3];
            var depNames = storeInfo.depStoreNames || [];
            var depStores = {};
            try {
                for (var depNames_2 = (e_6 = void 0, __values(depNames)), depNames_2_1 = depNames_2.next(); !depNames_2_1.done; depNames_2_1 = depNames_2.next()) {
                    var depName = depNames_2_1.value;
                    depStores[depName] = this.stores[this._getStoreKey(depName, storeOpts)];
                }
            }
            catch (e_6_1) { e_6 = { error: e_6_1 }; }
            finally {
                try {
                    if (depNames_2_1 && !depNames_2_1.done && (_b = depNames_2.return)) _b.call(depNames_2);
                }
                finally { if (e_6) throw e_6.error; }
            }
            var stateKey = storeInfo.options.stateKey;
            var initState = this.__initStates__
                ? stateKey
                    ? this.__initStates__[stateKey]
                    : this.__initStates__
                : undefined;
            this.stores[this._getStoreKey(storeKey_3, storeOpts)]._inject(this, storeInfo.Store, this._getStateKey(storeKey_3, stateKey, storeOpts), depStores, initState, storeInfo.options);
        }
        return depList;
    };
    AppStore.prototype._disposeStoreAndDeps = function (storeKey, store, storeOpts) {
        var e_7, _a;
        if (store.getRefCount() > 0)
            return;
        store.dispose();
        // delete this.stores[this._getStoreKey(storeKey, storeOpts)];
        this._deleteStore(storeKey, storeOpts);
        var depList = [storeKey];
        for (var i = 0; i < depList.length; i += 1) {
            var curStoreKey = depList[i];
            var storeInfo = this._storeRegisterMap[curStoreKey];
            var depNames = storeInfo.depStoreNames || [];
            var filterDepNames = [];
            try {
                for (var depNames_3 = (e_7 = void 0, __values(depNames)), depNames_3_1 = depNames_3.next(); !depNames_3_1.done; depNames_3_1 = depNames_3.next()) {
                    var depName = depNames_3_1.value;
                    var depStore = this.stores[this._getStoreKey(depName, storeOpts)];
                    if (!depStore)
                        continue;
                    depStore._decreaseRef();
                    if (depStore.getRefCount() === 0) {
                        depStore.dispose();
                        // delete this.stores[this._getStoreKey(depName, storeOpts)];
                        this._deleteStore(depName, storeOpts);
                        if (depList.indexOf(depName) === -1) {
                            filterDepNames.push(depName);
                        }
                    }
                }
            }
            catch (e_7_1) { e_7 = { error: e_7_1 }; }
            finally {
                try {
                    if (depNames_3_1 && !depNames_3_1.done && (_a = depNames_3.return)) _a.call(depNames_3);
                }
                finally { if (e_7) throw e_7.error; }
            }
            depList.splice.apply(depList, __spread([depList.length, 0], filterDepNames));
        }
    };
    // calculate the storeKey by the origin storeKey and storeOpts
    AppStore.prototype._getStoreKey = function (storeKey, storeOpts) {
        return storeKey;
    };
    // Extract the origin storeKey and storeOpts from the final store key.
    AppStore.prototype._parseStoreKey = function (finalStoreKey) {
        return [finalStoreKey, undefined];
    };
    // calculate the stateKey by the origin stateKey and storeOpts
    AppStore.prototype._getStateKey = function (storeKey, stateKey, storeOpts) {
        return stateKey;
    };
    AppStore.prototype._createStore = function (storeKey, store, storeOpts) {
        this.stores[this._getStoreKey(storeKey, storeOpts)] = store;
    };
    AppStore.prototype._deleteStore = function (storeKey, storeOpts) {
        delete this.stores[this._getStoreKey(storeKey, storeOpts)];
    };
    return AppStore;
}());
exports.default = AppStore;
AppStore.prototype[IS_APP_STORE] = true;
AppStore.isAppStore = function (maybeAppStore) {
    return !!(maybeAppStore && maybeAppStore[IS_APP_STORE]);
};
