"use strict";
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var LRUNode = /** @class */ (function () {
    function LRUNode(key, value, next, prev) {
        this.key = key;
        this.value = value;
        this.prev = prev;
        this.next = next;
    }
    return LRUNode;
}());
exports.LRUNode = LRUNode;
var LRU = /** @class */ (function () {
    //set default limit of 10 if limit is not passed.
    function LRU(limit, onExceedRemove) {
        if (limit === void 0) { limit = 10; }
        this.size = 0;
        this.limit = limit;
        this.cache = {};
        this.onExceedRemove = onExceedRemove;
    }
    // Write Node to head of LinkedList
    // update cache with Node key and Node reference
    LRU.prototype.put = function (key, value) {
        this.ensureLimit();
        if (!this.head) {
            this.head = this.tail = new LRUNode(key, value);
        }
        else {
            var node = new LRUNode(key, value, this.head);
            this.head.prev = node;
            this.head = node;
        }
        //Update the cache map
        this.cache[key] = this.head;
        this.size++;
    };
    LRU.prototype.take = function (key) {
        if (this.cache[key]) {
            var value = this.cache[key].value;
            // node removed from it's position and cache
            this.remove(key);
            return value;
        }
        return undefined;
    };
    // Get from cache map and make that node as new Head of LinkedList
    LRU.prototype.get = function (key) {
        if (this.cache[key]) {
            var value = this.cache[key].value;
            // node removed from it's position and cache
            this.remove(key);
            // write node again to the head of LinkedList to make it most recently used
            this.put(key, value);
            return value;
        }
        return undefined;
    };
    LRU.prototype.ensureLimit = function () {
        if (this.size >= this.limit) {
            var key = this.tail.key;
            this.remove(key);
            this.onExceedRemove && this.onExceedRemove(key);
        }
    };
    LRU.prototype.remove = function (key) {
        var node = this.cache[key];
        if (node.prev !== undefined) {
            node.prev.next = node.next;
        }
        else {
            this.head = node.next;
        }
        if (node.next !== undefined) {
            node.next.prev = node.prev;
        }
        else {
            this.tail = node.prev;
        }
        delete this.cache[key];
        this.size--;
    };
    LRU.prototype.clear = function () {
        this.head = undefined;
        this.tail = undefined;
        this.size = 0;
        this.cache = {};
    };
    // Invokes the callback function with every node of the chain and the index of the node.
    LRU.prototype.forEach = function (fn) {
        var node = this.head;
        var counter = 0;
        while (node) {
            fn(node, counter);
            node = node.next;
            counter++;
        }
    };
    // To iterate over LRU with a 'for...of' loop
    LRU.prototype[Symbol.iterator] = function () {
        var node;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    node = this.head;
                    _a.label = 1;
                case 1:
                    if (!node) return [3 /*break*/, 3];
                    return [4 /*yield*/, node];
                case 2:
                    _a.sent();
                    node = node.next;
                    return [3 /*break*/, 1];
                case 3: return [2 /*return*/];
            }
        });
    };
    return LRU;
}());
exports.default = LRU;
