import { Emitter, Disposable, CompositeDisposable, DisposableLike } from "event-kit";
import DispatchParent from "./DispatchParent";
import DispatchItem from "./DispatchItem";
import { StoreBaseConstructor, StoreDeclarerOptions } from "./StoreDeclarer";
export declare function eventListener(target: StoreBase<any>, propertyKey: string, descriptor: PropertyDescriptor): any;
export declare function reducer(target: StoreBase<any>, propertyKey: string, descriptor: PropertyDescriptor): void;
export declare function invoker(target: any, propertyKey: string, descriptor: PropertyDescriptor): void;
export default class StoreBase<StateT> implements DispatchItem {
    state: StateT;
    emitter: Emitter<{
        [key: string]: any;
    }, {}>;
    disposables: CompositeDisposable;
    _inWillUpdate: boolean;
    _willUpdateStates: any[];
    _appStore: DispatchParent | undefined;
    _stateKey: string | undefined;
    _refCount: number;
    __enableUpdate: boolean;
    _hasUpdate: boolean;
    mapKey?: string;
    listIndex?: number;
    [storeKey: string]: any;
    static isStore: (store: any) => boolean;
    /**
     * Inject the store's dependency stores into this store, It will be invoked before init
     * @param depStores: the store's dependencies, the dependency stores will be injected to this store
     */
    _inject(appStore: DispatchParent | undefined, StoreBuilder: StoreBaseConstructor<any>, stateKey?: string, depStores?: {
        [storeKey: string]: DispatchItem;
    }, initState?: any, options?: StoreDeclarerOptions): void;
    _handleUpdate: (state: any) => void;
    _init(): void | Promise<void>;
    init(): void | Promise<void>;
    _disableUpdate(): void;
    _enableUpdate: () => void;
    setState(state: any): void;
    onDidUpdate(callback: (value: StateT) => void): Disposable;
    onWillUpdate(callback: (value: StateT) => void): Disposable;
    observe(callback: (value: StateT) => void): Disposable;
    addDisposable(item: DisposableLike): void;
    dispose(): void;
    getState(): StateT;
    _addRef(): void;
    _decreaseRef(): void;
    getRefCount(): number;
}
