"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var event_kit_1 = require("event-kit");
var IS_STORE = "@@__FLUX_STORE__@@";
function eventListener(target, propertyKey, descriptor) {
    if (!target._evs) {
        target._evs = [];
    }
    target._evs.push(propertyKey);
    return target[propertyKey];
}
exports.eventListener = eventListener;
function reducer(target, propertyKey, descriptor) {
    var originalMethod = descriptor.value;
    descriptor.value = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this._disableUpdate();
        var retVal = originalMethod.apply(this, args);
        if (retVal && retVal.finally) {
            retVal.finally(this._enableUpdate);
        }
        else {
            this._enableUpdate();
        }
    };
}
exports.reducer = reducer;
function invoker(target, propertyKey, descriptor) {
    var originalMethod = descriptor.value;
    if (!target._invokers) {
        target._invokers = [];
    }
    target._invokers.push(propertyKey);
    descriptor.value = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this._disableUpdate();
        var retVal = originalMethod.apply(this, args);
        if (retVal && retVal.finally) {
            return retVal.finally(this._enableUpdate);
        }
        else {
            this._enableUpdate();
            return retVal;
        }
    };
}
exports.invoker = invoker;
var StoreBase = /** @class */ (function () {
    function StoreBase() {
        var _this = this;
        this.state = {};
        this.emitter = new event_kit_1.Emitter();
        this.disposables = new event_kit_1.CompositeDisposable();
        this._inWillUpdate = false;
        this._willUpdateStates = [];
        this._refCount = 0;
        this.__enableUpdate = true;
        this._hasUpdate = false;
        this._handleUpdate = function (state) {
            var _a;
            var stateKey = _this._stateKey;
            if (!_this._appStore)
                return;
            if (stateKey) {
                _this._appStore.setState((_a = {}, _a[stateKey] = state, _a));
            }
            else {
                _this._appStore.setState(state);
            }
        };
        this._enableUpdate = function () {
            _this.__enableUpdate = true;
            if (_this._hasUpdate) {
                _this._hasUpdate = false;
                _this.emitter.emit("did-update", _this.state);
            }
        };
    }
    // static innerStores;
    /**
     * Inject the store's dependency stores into this store, It will be invoked before init
     * @param depStores: the store's dependencies, the dependency stores will be injected to this store
     */
    StoreBase.prototype._inject = function (appStore, StoreBuilder, stateKey, depStores, initState, options) {
        this._appStore = appStore;
        this._stateKey = stateKey;
        if (depStores) {
            for (var storeKey in depStores) {
                this[storeKey] = depStores[storeKey];
            }
        }
        // Observe this store's state and send the state to appStore
        if (initState) {
            this.state = initState;
            this.addDisposable(this.onDidUpdate(this._handleUpdate));
        }
        else {
            this.addDisposable(this.observe(this._handleUpdate));
        }
    };
    StoreBase.prototype._init = function () {
        // Before init, we will disable update
        this._disableUpdate();
        var retVal = this.init();
        // Check the return value is Promise-like or not
        if (retVal && retVal.finally) {
            return retVal.finally(this._enableUpdate);
        }
        else {
            this._enableUpdate();
        }
    };
    StoreBase.prototype.init = function () { };
    StoreBase.prototype._disableUpdate = function () {
        this.__enableUpdate = false;
    };
    StoreBase.prototype.setState = function (state) {
        // 当will-update，将状态保存到缓存队列中
        if (this._inWillUpdate) {
            this._willUpdateStates.push(state);
            return;
        }
        // Make the update delay to next tick that can collect many update into one operation.
        var nextState = Object.assign({}, this.state, state);
        this._inWillUpdate = true;
        this.emitter.emit("will-update", nextState);
        this._inWillUpdate = false;
        if (this._willUpdateStates.length > 0) {
            this.state = this._willUpdateStates.reduce(function (allState, state) { return Object.assign(allState, state); }, nextState);
            this._willUpdateStates = [];
        }
        else {
            this.state = nextState;
        }
        // Send update notification.
        if (this.__enableUpdate) {
            this.emitter.emit("did-update", this.state);
        }
        else {
            this._hasUpdate = true;
        }
    };
    StoreBase.prototype.onDidUpdate = function (callback) {
        return this.emitter.on("did-update", callback);
    };
    StoreBase.prototype.onWillUpdate = function (callback) {
        return this.emitter.on("will-update", callback);
    };
    StoreBase.prototype.observe = function (callback) {
        callback(this.state);
        return this.emitter.on("did-update", callback);
    };
    StoreBase.prototype.addDisposable = function (item) {
        this.disposables.add(item);
    };
    StoreBase.prototype.dispose = function () {
        this.disposables.dispose();
        this.emitter.dispose();
        this._handleUpdate(undefined);
    };
    StoreBase.prototype.getState = function () {
        return this.state;
    };
    // Add the ref count and avoid recycle this store
    StoreBase.prototype._addRef = function () {
        this._refCount += 1;
    };
    // Decrease the store's ref count, if this ref count decrease to 0, this store  will be disposed.
    StoreBase.prototype._decreaseRef = function () {
        this._refCount -= 1;
    };
    StoreBase.prototype.getRefCount = function () {
        return this._refCount;
    };
    __decorate([
        eventListener
    ], StoreBase.prototype, "onDidUpdate", null);
    __decorate([
        eventListener
    ], StoreBase.prototype, "onWillUpdate", null);
    __decorate([
        eventListener
    ], StoreBase.prototype, "observe", null);
    return StoreBase;
}());
exports.default = StoreBase;
StoreBase.prototype[IS_STORE] = true;
StoreBase.isStore = function (maybeStore) {
    return !!(maybeStore && maybeStore[IS_STORE]);
};
