import StoreBase from "./StoreBase";
import StoreList from "./StoreList";
import StoreMap from "./StoreMap";
import RecycleStrategy from "./RecycleStrategy";
import AppStore from "./AppStore";
export interface IGenericOptions {
    stateKey?: string;
    storeKey?: string;
    lifetime?: "static" | "dynamic";
    [key: string]: any;
}
export interface StoreBaseConstructor<T> {
    new (): StoreBase<T>;
}
export interface StoreListConstructor<T> {
    new (): StoreList<T>;
}
export interface StoreMapConstructor<T> {
    new (): StoreMap<T>;
}
export interface StoreDeclarerOptions extends IGenericOptions {
    storeKey?: string;
    stateKey?: string;
    [key: string]: any;
}
declare const IS_STORE = "@@__STORE_ITEM__@@";
export declare class StoreDeclarer<T> {
    Store: StoreBaseConstructor<T>;
    depStoreNames: string[] | undefined;
    options: StoreDeclarerOptions | undefined;
    constructor(Store: StoreBaseConstructor<T>, depStoreNames?: string[] | StoreDeclarerOptions, options?: StoreDeclarerOptions);
    create(appStore: AppStore): StoreBase<T>;
    [IS_STORE]: boolean;
    static isStore(maybeStore: any): boolean;
}
export declare function declareStore<T>(Store: StoreBaseConstructor<T>, depStoreNames?: string[] | StoreDeclarerOptions, options?: StoreDeclarerOptions): StoreDeclarer<T>;
export interface StoreListDeclarerOptions extends IGenericOptions {
    storeKey?: string;
    stateKey?: string;
    size?: number;
    StoreList?: StoreListConstructor<any>;
    [key: string]: any;
}
declare const IS_STORE_LIST = "@@__STORE_LIST__@@";
export declare class StoreListDeclarer<T> {
    Store: StoreBaseConstructor<T>;
    depStoreNames: string[] | undefined;
    options: StoreListDeclarerOptions | undefined;
    constructor(Store: StoreBaseConstructor<T>, depStoreNames?: string[] | StoreListDeclarerOptions, options?: StoreListDeclarerOptions);
    create(appStore: AppStore): StoreList<T>;
    [IS_STORE_LIST]: boolean;
    static isStoreList(maybeList: any): boolean;
}
export declare function declareStoreList<T>(Store: StoreBaseConstructor<T>, depStoreNames?: string[] | StoreListDeclarerOptions, options?: StoreListDeclarerOptions): StoreListDeclarer<T>;
export interface StoreMapDeclarerOptions extends IGenericOptions {
    storeKey?: string;
    stateKey?: string;
    keys?: string[];
    recycleStrategy?: RecycleStrategy;
    cacheLimit?: number;
    StoreMap?: StoreMapConstructor<any>;
    [key: string]: any;
}
declare const IS_STORE_MAP = "@@__STORE_MAP__@@";
export declare class StoreMapDeclarer<T> {
    Store: StoreBaseConstructor<T>;
    depStoreNames: string[] | undefined;
    options: StoreMapDeclarerOptions | undefined;
    constructor(Store: StoreBaseConstructor<T>, depStoreNames?: string[] | StoreMapDeclarerOptions, options?: StoreMapDeclarerOptions);
    create(appStore: AppStore): StoreMap<T>;
    [IS_STORE_MAP]: boolean;
    static isStoreMap(maybeMap: any): boolean;
}
export declare function declareStoreMap<T>(Store: StoreBaseConstructor<T>, depStoreNames?: string[] | StoreMapDeclarerOptions, options?: StoreMapDeclarerOptions): StoreMapDeclarer<T>;
export declare type AnyStoreDeclarer = StoreDeclarer<any> | StoreListDeclarer<any> | StoreMapDeclarer<any>;
export {};
