"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var StoreList = /** @class */ (function () {
    function StoreList() {
        this.length = 0;
        this.storeArray = [];
        this._depStores = {};
        this._refCount = 0;
        this.state = {};
    }
    StoreList.prototype._init = function () {
        if (this._options.size) {
            return Promise.all([this.setSize(this._options.size), this.init()]);
        }
        return this.init();
    };
    StoreList.prototype.init = function () { };
    StoreList.prototype._inject = function (appStore, StoreBuilder, stateKey, depStores, initState, options) {
        this._appStore = appStore;
        this._stateKey = stateKey;
        if (!stateKey)
            console.error("StoreList can not let stateKey to null");
        this._StoreBuilder = StoreBuilder;
        this._options = options;
        if (depStores) {
            this._depStores = depStores;
        }
        if (initState) {
            this.__initStates__ = initState;
            this.state = initState;
        }
    };
    StoreList.prototype.setInitStates = function (initStates) {
        this.__initStates__ = initStates;
    };
    StoreList.prototype.setSize = function (count) {
        if (this.length === count)
            return;
        if (this.length < count) {
            var initList = [];
            for (var i = this.length; i < count; ++i) {
                var newStore = new this._StoreBuilder();
                newStore.listIndex = i;
                var initState = this.__initStates__ ? this.__initStates__[i] : undefined;
                newStore._inject(this, this._StoreBuilder, i.toString(), this._depStores, initState, {});
                this.storeArray.push(newStore);
                // newStore._init();
                initList.push(newStore);
            }
            this.length = count;
            return Promise.all(initList.map(function (store) { return store._init(); }));
        }
        else {
            for (var i = count; i < this.length; ++i) {
                this.storeArray[i].dispose();
            }
            this.storeArray.splice(count, this.length - count);
            this.length = count;
            return;
        }
    };
    StoreList.prototype.setState = function (state) {
        var _a;
        this.state = __assign(__assign({}, this.state), state);
        if (this._stateKey && this._appStore) {
            this._appStore.setState((_a = {}, _a[this._stateKey] = this.state, _a));
        }
    };
    StoreList.prototype.dispose = function () {
        var _a;
        this.setSize(0);
        this.state = {};
        if (this._stateKey && this._appStore) {
            this._appStore.setState((_a = {}, _a[this._stateKey] = undefined, _a));
        }
    };
    StoreList.prototype.forEach = function (callback) {
        return this.storeArray.forEach(callback);
    };
    StoreList.prototype.map = function (callback) {
        return this.storeArray.map(callback);
    };
    StoreList.prototype.filter = function (callback) {
        return this.storeArray.filter(callback);
    };
    StoreList.prototype.get = function (index) {
        return this.storeArray[index];
    };
    StoreList.prototype.slice = function (begin, end) {
        return this.storeArray.slice(begin, end);
    };
    StoreList.prototype.indexOf = function (item) {
        return this.storeArray.indexOf(item);
    };
    StoreList.prototype._addRef = function () {
        this._refCount += 1;
    };
    StoreList.prototype._decreaseRef = function () {
        this._refCount -= 1;
    };
    StoreList.prototype.getRefCount = function () {
        return this._refCount;
    };
    return StoreList;
}());
exports.default = StoreList;
