"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
var event_kit_1 = require("event-kit");
var RecycleStrategy_1 = require("./RecycleStrategy");
var LRU_1 = require("./LRU");
var OperateMode;
(function (OperateMode) {
    OperateMode[OperateMode["None"] = 0] = "None";
    OperateMode[OperateMode["Direct"] = 1] = "Direct";
    OperateMode[OperateMode["RefCount"] = 2] = "RefCount";
})(OperateMode = exports.OperateMode || (exports.OperateMode = {}));
var OperateModeSwitch = /** @class */ (function () {
    function OperateModeSwitch() {
        this.operateMode = OperateMode.None;
    }
    OperateModeSwitch.prototype.enterRefCountMode = function () {
        if (this.operateMode === OperateMode.Direct) {
            throw new Error("\n        You request store that use direct mode, so requesting store by \"request\" method is not allowed. \n        Try to remove all the \"add\" invocation and \"keys\" declarer options.\n      ");
        }
        this.operateMode = OperateMode.RefCount;
    };
    OperateModeSwitch.prototype.enterDirectMode = function () {
        if (this.operateMode === OperateMode.RefCount) {
            throw new Error("You request store that use reference count, so requesting store by \"add\" and \"delete\" is not allowed.");
        }
        this.operateMode = OperateMode.Direct;
    };
    return OperateModeSwitch;
}());
exports.OperateModeSwitch = OperateModeSwitch;
var StoreMap = /** @class */ (function () {
    function StoreMap() {
        this.storeMap = new Map();
        this._depStores = {};
        this._refCount = 0;
        this.emitter = new event_kit_1.Emitter();
        this.state = {};
        this._keyRefs = {};
        this.disposables = new event_kit_1.CompositeDisposable();
        this.operateModeSwitch = new OperateModeSwitch();
    }
    StoreMap.prototype._init = function () {
        var _this = this;
        var keys = this._options.keys;
        if (keys && Array.isArray(keys)) {
            return Promise.all(__spread(keys.map(function (key) { return _this.add(key); }), [this.init()]));
        }
        return this.init();
    };
    StoreMap.prototype.init = function () { };
    StoreMap.prototype._inject = function (appStore, StoreBuilder, stateKey, depStores, initState, options) {
        this._appStore = appStore;
        this._stateKey = stateKey;
        if (!stateKey)
            console.error("StoreList can not let stateKey to null");
        this._StoreBuilder = StoreBuilder;
        this._options = options;
        if (depStores) {
            this._depStores = depStores;
        }
        if (initState) {
            this.__initStates__ = initState;
            this.state = initState;
        }
        if (options && options.recycleStrategy != null) {
            this.setRecycleStrategy(options.recycleStrategy, { cacheLimit: options.cacheLimit });
        }
    };
    // Dispose all the sub stores that reference count is 0
    StoreMap.prototype._disposeSubStores = function () {
        var keyRefs = this._keyRefs;
        for (var storeKey in keyRefs) {
            if (keyRefs[storeKey] === 0 && this.storeMap.has(storeKey)) {
                this._deleteOne(storeKey);
            }
        }
    };
    StoreMap.prototype.onDidChangeRS = function (callback) {
        return this.emitter.on("did-change-rs", callback);
    };
    StoreMap.prototype.observeRS = function (callback) {
        if (this._recycleStrategy != null) {
            callback(this._recycleStrategy);
        }
        return this.onDidChangeRS(callback);
    };
    StoreMap.prototype.setRecycleStrategy = function (recycleStrategy, options) {
        var _this = this;
        if (this._recycleStrategy !== recycleStrategy) {
            this._recycleStrategy = recycleStrategy;
            this._keyCache = undefined;
            if (recycleStrategy === RecycleStrategy_1.default.Urgent) {
                this._disposeSubStores();
            }
            else if (recycleStrategy === RecycleStrategy_1.default.Cache) {
                this._keyCache = new LRU_1.default(options && options.cacheLimit, function (removeKey) {
                    _this._deleteOne(removeKey);
                });
            }
            this.emitter.emit("did-change-rs", recycleStrategy);
        }
    };
    StoreMap.prototype.setInitStates = function (initStates) {
        this.__initStates__ = initStates;
    };
    StoreMap.prototype.requestStore = function (storeKey) {
        this.operateModeSwitch.enterRefCountMode();
        if (this._keyRefs[storeKey]) {
            this._keyRefs[storeKey] += 1;
        }
        else {
            this._keyRefs[storeKey] = 1;
            // If the cache has not this key, then we need create the new store.
            if (!this._keyCache || !this._keyCache.take(storeKey)) {
                this._addOne(storeKey);
            }
        }
    };
    StoreMap.prototype.releaseStore = function (storeKey) {
        this._keyRefs[storeKey] -= 1;
        if (this._keyRefs[storeKey] === 0) {
            if (this._recycleStrategy === undefined &&
                this._appStore._recycleStrategy === RecycleStrategy_1.default.Urgent) {
                this._deleteOne(storeKey);
            }
            else if (this._recycleStrategy === RecycleStrategy_1.default.Urgent) {
                this._deleteOne(storeKey);
            }
            else if (this._recycleStrategy === RecycleStrategy_1.default.Cache) {
                this._keyCache.put(storeKey, storeKey);
            }
        }
    };
    // Request the store that storeKey is `keys`, use the reference count
    StoreMap.prototype.request = function (keys) {
        var e_1, _a;
        var _this = this;
        if (!Array.isArray(keys)) {
            keys = [keys];
        }
        try {
            for (var keys_1 = __values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                var key = keys_1_1.value;
                this.requestStore(key);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var disposable = {
            dispose: function () {
                var e_2, _a;
                try {
                    for (var keys_2 = __values(keys), keys_2_1 = keys_2.next(); !keys_2_1.done; keys_2_1 = keys_2.next()) {
                        var key = keys_2_1.value;
                        _this.releaseStore(key);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (keys_2_1 && !keys_2_1.done && (_a = keys_2.return)) _a.call(keys_2);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
        };
        this.disposables.add(disposable);
        return disposable;
    };
    StoreMap.prototype._addOne = function (key) {
        if (this.storeMap.has(key))
            return;
        var newStore = new this._StoreBuilder();
        newStore.mapKey = key;
        var initState = this.__initStates__ ? this.__initStates__[key] : undefined;
        newStore._inject(this, this._StoreBuilder, key, this._depStores, initState, {});
        this.storeMap.set(key, newStore);
        return newStore._init();
    };
    StoreMap.prototype._deleteOne = function (key) {
        var store = this.storeMap.get(key);
        store && store.dispose();
        this.storeMap.delete(key);
    };
    StoreMap.prototype.add = function (key) {
        var e_3, _a;
        this.operateModeSwitch.enterDirectMode();
        if (Array.isArray(key)) {
            try {
                for (var key_1 = __values(key), key_1_1 = key_1.next(); !key_1_1.done; key_1_1 = key_1.next()) {
                    var k = key_1_1.value;
                    this._addOne(k);
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (key_1_1 && !key_1_1.done && (_a = key_1.return)) _a.call(key_1);
                }
                finally { if (e_3) throw e_3.error; }
            }
            return;
        }
        this._addOne(key);
    };
    StoreMap.prototype.delete = function (key) {
        var e_4, _a;
        if (Array.isArray(key)) {
            try {
                for (var key_2 = __values(key), key_2_1 = key_2.next(); !key_2_1.done; key_2_1 = key_2.next()) {
                    var k = key_2_1.value;
                    this._deleteOne(k);
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (key_2_1 && !key_2_1.done && (_a = key_2.return)) _a.call(key_2);
                }
                finally { if (e_4) throw e_4.error; }
            }
            return;
        }
        this._deleteOne(key);
    };
    StoreMap.prototype.clear = function () {
        var e_5, _a;
        var stores = this.storeMap.values();
        try {
            for (var stores_1 = __values(stores), stores_1_1 = stores_1.next(); !stores_1_1.done; stores_1_1 = stores_1.next()) {
                var store = stores_1_1.value;
                store.dispose();
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (stores_1_1 && !stores_1_1.done && (_a = stores_1.return)) _a.call(stores_1);
            }
            finally { if (e_5) throw e_5.error; }
        }
        this.storeMap.clear();
    };
    StoreMap.prototype.setState = function (state) {
        var _a;
        this.state = __assign(__assign({}, this.state), state);
        if (this._stateKey && this._appStore) {
            this._appStore.setState((_a = {}, _a[this._stateKey] = this.state, _a));
        }
    };
    StoreMap.prototype.dispose = function () {
        var _a;
        this.clear();
        this.disposables.dispose();
        this.state = {};
        this.emitter.dispose();
        if (this._stateKey && this._appStore) {
            this._appStore.setState((_a = {}, _a[this._stateKey] = undefined, _a));
        }
    };
    StoreMap.prototype.forEach = function (callback) {
        return this.storeMap.forEach(callback);
    };
    StoreMap.prototype.get = function (key) {
        return this.storeMap.get(key);
    };
    StoreMap.prototype.has = function (key) {
        return this.storeMap.has(key);
    };
    StoreMap.prototype.keys = function () {
        return this.storeMap.keys();
    };
    StoreMap.prototype.values = function () {
        return this.storeMap.values();
    };
    StoreMap.prototype.entries = function () {
        return this.storeMap.entries();
    };
    StoreMap.prototype._addRef = function () {
        this._refCount += 1;
    };
    StoreMap.prototype._decreaseRef = function () {
        this._refCount -= 1;
    };
    StoreMap.prototype.getRefCount = function () {
        return this._refCount;
    };
    return StoreMap;
}());
exports.default = StoreMap;
